﻿using CaseAccountingContracts.BindingModels;
using CaseAccountingContracts.BusinessLogicContracts;
using CaseAccountingContracts.SearchModels;
using CaseAccountingContracts.StoragesContracts;
using CaseAccountingContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CaseAccountingBusinessLogic.BusinessLogics
{
    public class LawyerLogic : ILawyerLogic
    {
        private readonly ILogger _logger;
        private readonly ILawyerStorage _lawyerStorage;

        public LawyerLogic(ILogger<LawyerLogic> logger, ILawyerStorage lawyerStorage)
        {
            _logger = logger ?? throw new ArgumentNullException(nameof(logger));
            _lawyerStorage = lawyerStorage ?? throw new ArgumentNullException(nameof(lawyerStorage));
        }

        public bool Create(LawyerBindingModel model)
        {
            CheckModel(model);
            if (_lawyerStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(LawyerBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_lawyerStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        public LawyerViewModel? ReadElement(LawyerSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id:{ Id}", model.Id);
            var element = _lawyerStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public List<LawyerViewModel>? ReadList(LawyerSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id:{ Id}", model?.Id);
            var list = model == null ? _lawyerStorage.GetFullList() : _lawyerStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public bool Update(LawyerBindingModel model)
        {
            CheckModel(model);
            if (_lawyerStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(LawyerBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.UserId < 0)
            {
                throw new ArgumentNullException("Некорректный идентификатор пользователя",
                    nameof(model.UserId));
            }
            if (model.SpecializationId < 0) 
            {
                throw new ArgumentNullException("Некорректный идентификатор специализации",
                    nameof(model.SpecializationId));
            }
            if (model.Experience < 0)
            {
                throw new ArgumentNullException("Некорректно указан опыт работы",
                    nameof(model.Experience));
            }
            if (model.Name == string.Empty) 
            {
                throw new ArgumentNullException("Некорректное имя юриста",
                    nameof(model.Name));
            }
            if (model.Surname == string.Empty)
            {
                throw new ArgumentNullException("Некорректная фамилия юриста",
                    nameof(model.Surname));
            }
        }
    }
}
